@@ -204,26 +204,16 @@ namespace af
                       const dim_t d3, const dtype ty=f32);
 
     /**
-        \defgroup data_func_setseed setSeed
-        Set the seed for the random number generator
-
-
         \param[in] seed is a 64 bit unsigned integer
 
-        \ingroup data_mat
-        \ingroup arrayfire_func
+        \ingroup data_func_setseed
     */
     AFAPI void setSeed(const uintl seed);
 
     /**
-        \defgroup data_func_getseed getSeed
-        Get the seed for the random number generator
-
-
         \returns seed which is a 64 bit unsigned integer
 
-        \ingroup data_mat
-        \ingroup arrayfire_func
+        \ingroup data_func_getseed
     */
     AFAPI uintl getSeed();
 
@@ -489,6 +479,64 @@ namespace af
     */
     AFAPI array upper(const array &in, bool is_unit_diag=false);
 
+#if AF_API_VERSION >= 31
+    /**
+       \param[in]  cond is the conditional array
+       \param[in]  a is the array containing elements from the true part of the condition
+       \param[in]  b is the array containing elements from the false part of the condition
+       \return  the output containing elements of \p a when \p cond is true else elements from \p b
+
+       \ingroup data_func_select
+    */
+    AFAPI array select(const array &cond, const array  &a, const array  &b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[in]  cond is the conditional array
+       \param[in]  a is the array containing elements from the true part of the condition
+       \param[in]  b is a scalar assigned to \p out when \p cond is false
+       \return  the output containing elements of \p a when \p cond is true else the value \p b
+
+       \ingroup data_func_select
+    */
+    AFAPI array select(const array &cond, const array  &a, const double &b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[in]  cond is the conditional array
+       \param[in]  a is a scalar assigned to \p out when \p cond is true
+       \param[in]  b is the array containing elements from the false part of the condition
+       \return  the output containing the value \p a when \p cond is true else elements from \p b
+
+       \ingroup data_func_select
+    */
+    AFAPI array select(const array &cond, const double &a, const array  &b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[inout]  a is the array whose values are replaced with values from \p b when \p cond is false
+       \param[in]  cond is the conditional array
+       \param[in]  b is the array containing elements which replace elements in \p a when \p cond is false
+
+       \ingroup data_func_replace
+    */
+    AFAPI void replace(array &a, const array  &cond, const array  &b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[inout]  a is the array whose values are replaced with values from \p b when \p cond is false
+       \param[in]  cond is the conditional array
+       \param[in]  b is value that replaces elements in \p a when \p cond is false
+
+       \ingroup data_func_replace
+    */
+    AFAPI void replace(array &a, const array  &cond, const double &b);
+#endif
+
     /**
       @}
     */
@@ -596,26 +644,16 @@ extern "C" {
     AFAPI af_err af_randn(af_array *out, const unsigned ndims, const dim_t * const dims, const af_dtype type);
 
     /**
-        \defgroup data_func_setseed setSeed
-        Set the seed for the random number generator
-
-
         \param[in] seed is a 64 bit unsigned integer
 
-        \ingroup data_mat
-        \ingroup arrayfire_func
+        \ingroup data_func_setseed
     */
     AFAPI af_err af_set_seed(const uintl seed);
 
     /**
-        \defgroup data_func_getseed getSeed
-        Get the seed for the random number generator
-
-
         \param[out] seed which is a 64 bit unsigned integer
 
-        \ingroup data_mat
-        \ingroup arrayfire_func
+        \ingroup data_func_getseed
     */
     AFAPI af_err af_get_seed(uintl *seed);
 
@@ -759,6 +797,65 @@ extern "C" {
     /**
       @}
     */
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[out] out is the output containing elements of \p a when \p cond is true else elements from \p b
+       \param[in]  cond is the conditional array
+       \param[in]  a is the array containing elements from the true part of the condition
+       \param[in]  b is the array containing elements from the false part of the condition
+
+       \ingroup data_func_select
+    */
+    AFAPI af_err af_select(af_array *out, const af_array cond, const af_array a, const af_array b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[out] out is the output containing elements of \p a when \p cond is true else elements from \p b
+       \param[in]  cond is the conditional array
+       \param[in]  a is the array containing elements from the true part of the condition
+       \param[in]  b is a scalar assigned to \p out when \p cond is false
+
+       \ingroup data_func_select
+    */
+    AFAPI af_err af_select_scalar_r(af_array *out, const af_array cond, const af_array a, const double b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[out] out is the output containing elements of \p a when \p cond is true else elements from \p b
+       \param[in]  cond is the conditional array
+       \param[in]  a is a scalar assigned to \p out when \p cond is true
+       \param[in]  b is the array containing elements from the false part of the condition
+
+       \ingroup data_func_select
+    */
+    AFAPI af_err af_select_scalar_l(af_array *out, const af_array cond, const double a, const af_array b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[inout] a is the array whose values are replaced by \p b when \p cond is false
+       \param[in]  cond is the conditional array
+       \param[in]  b is the array containing elements that replaces elements of a where \p cond is false
+
+       \ingroup data_func_replace
+    */
+    AFAPI af_err af_replace(af_array a, const af_array cond, const af_array b);
+#endif
+
+#if AF_API_VERSION >= 31
+    /**
+       \param[inout] a is the array whose values are replaced by \p b when \p cond is false
+       \param[in]  cond is the conditional array
+       \param[in]  b is the scalar that replaces the false parts of \p a
+
+       \ingroup data_func_replace
+    */
+    AFAPI af_err af_replace_scalar(af_array a, const af_array cond, const double b);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
