@@ -8,16 +8,14 @@
  ********************************************************/
 
 #include <af/exception.h>
+#include <defines.hpp>
 
 #define AF_THROW(fn) do {                               \
         af_err __err = fn;                              \
         if (__err == AF_SUCCESS) break;                 \
-        throw af::exception(__FILE__, __LINE__, __err); \
+        throw af::exception(__AF_FILENAME__, __LINE__, __err); \
     } while(0)
 
-#define AF_THROW_MSG(__msg, __err) do {                         \
-        if (__err == AF_SUCCESS) break;                         \
-        throw af::exception(__msg, __FILE__, __LINE__, __err);  \
-    } while(0);
-
-#define THROW(__err) throw af::exception(__FILE__, __LINE__, __err)
+#define AF_THROW_ERR(__msg, __err) do {                         \
+        throw af::exception(__msg, __AF_FILENAME__, __LINE__, __err);  \
+    } while(0)
