@@ -34,6 +34,8 @@ INSTANTIATE(float)
 INSTANTIATE(int)
 INSTANTIATE(uint)
 INSTANTIATE(uchar)
+INSTANTIATE(short)
+INSTANTIATE(ushort)
 
 }
 
