@@ -14,9 +14,6 @@ namespace cpu
     uint abs(uint val) { return val; }
     uchar abs(uchar val) { return val; }
     uintl abs(uintl val) { return val; }
-#if !(defined(OS_WIN) || (defined(ARCH_32) && defined(OS_LNX)))  // Not(Windows or Tegra)
-    size_t abs(size_t val) { return val; }
-#endif
 
     cfloat  scalar(float val)
     {
