@@ -40,7 +40,9 @@ class InteropManager
         ~InteropManager();
         cudaGraphicsResource* getBufferResource(const fg::Image* handle);
         cudaGraphicsResource* getBufferResource(const fg::Plot* handle);
+        cudaGraphicsResource* getBufferResource(const fg::Plot3* handle);
         cudaGraphicsResource* getBufferResource(const fg::Histogram* handle);
+        cudaGraphicsResource* getBufferResource(const fg::Surface* handle);
 
     protected:
         InteropManager() {}
