@@ -18,6 +18,10 @@ namespace opencl
     INSTANTIATE(af_notzero_t, cdouble, uint)
     INSTANTIATE(af_notzero_t, int    , uint)
     INSTANTIATE(af_notzero_t, uint   , uint)
+    INSTANTIATE(af_notzero_t, intl   , uint)
+    INSTANTIATE(af_notzero_t, uintl  , uint)
     INSTANTIATE(af_notzero_t, char   , uint)
     INSTANTIATE(af_notzero_t, uchar  , uint)
+    INSTANTIATE(af_notzero_t, short  , uint)
+    INSTANTIATE(af_notzero_t, ushort , uint)
 }
