@@ -12,7 +12,7 @@
 namespace opencl
 {
 
-template<typename inType, typename outType>
+template<typename inType, typename outType, bool isLinear>
 Array<outType> histogram(const Array<inType> &in, const unsigned &nbins, const double &minval, const double &maxval);
 
 }
