@@ -39,5 +39,9 @@ INSTANTIATE(char  )
 INSTANTIATE(int   )
 INSTANTIATE(uint  )
 INSTANTIATE(uchar )
+INSTANTIATE(short )
+INSTANTIATE(ushort)
+INSTANTIATE(intl  )
+INSTANTIATE(uintl )
 
 }
