@@ -16,11 +16,11 @@ namespace opencl
 
     cl::Buffer *bufferAlloc(const size_t &bytes);
     void bufferFree(cl::Buffer *buf);
-    void bufferUnlink(cl::Buffer *ptr);
 
     template<typename T> T *memAlloc(const size_t &elements);
     template<typename T> void memFree(T *ptr);
-    template<typename T> void memUnlink(T *ptr);
+    template<typename T> void memPop(const T *ptr);
+    template<typename T> void memPush(const T *ptr);
 
     template<typename T> T* pinnedAlloc(const size_t &elements);
     template<typename T> void pinnedFree(T* ptr);
