@@ -35,7 +35,7 @@ class FFTConvolveLarge : public ::testing::Test
 };
 
 // create a list of types to be tested
-typedef ::testing::Types<cfloat, cdouble, float, double, int, uint, char, uchar> TestTypes;
+typedef ::testing::Types<cfloat, cdouble, float, double, int, uint, char, uchar, intl, uintl> TestTypes;
 typedef ::testing::Types<float, double> TestTypesLarge;
 
 // register the type list
@@ -651,3 +651,42 @@ TEST(Padding, fftConvolve2)
         ASSERT_EQ(max<double>(abs(c - d)) < 1E-5, true);
     }
 }
+
+TEST(FFTConvolve1, Interleaved)
+{
+    array a = randu(100, 1, 2);
+    array b = randu(20, 3);
+    array c = fftConvolve1(a, b);
+
+    for (int ii = 0; ii < 2; ii++) {
+        array c_ii = c(span, span, ii);
+        array d = fftConvolve1(a(span, 0, ii), b);
+        ASSERT_EQ(max<double>(abs(c_ii - d)) < 1E-5, true);
+    }
+}
+
+TEST(FFTConvolve2, Interleaved)
+{
+    array a = randu(100, 100, 2);
+    array b = randu(5, 5, 1, 3);
+    array c = fftConvolve2(a, b);
+
+    for (int ii = 0; ii < 3; ii++) {
+        array c_ii = c(span, span, span, ii);
+        array d = fftConvolve2(a, b(span, span, 0, ii));
+        ASSERT_EQ(max<double>(abs(c_ii - d)) < 1E-5, true);
+    }
+}
+
+TEST(FFTConvolve2, Interleaved2)
+{
+    array a = randu(100, 100, 2);
+    array b = randu(5, 5, 2, 3);
+    array c = fftConvolve2(a, b);
+
+    for (int ii = 0; ii < 3; ii++) {
+        array c_ii = c(span, span, span, ii);
+        array d = fftConvolve2(a, b(span, span, span, ii));
+        ASSERT_EQ(max<double>(abs(c_ii - d)) < 1E-5, true);
+    }
+}
